@@ -5,12 +5,11 @@ from itertools import groupby
 
 from .api import SquadApi
 from squad_client.exceptions import InvalidSquadObject
-from squad_client.utils import first, parse_test_name, parse_metric_name
+from squad_client.utils import first, parse_test_name, parse_metric_name, to_json
 from squad_client import settings
 
 
-logger = logging.getLogger('models')
-logger.setLevel(logging.DEBUG)
+logger = logging.getLogger()
 
 DEFAULT_COUNT = settings.DEFAULT_NUM_OF_OBJECTS
 ALL = -1
@@ -21,6 +20,11 @@ class SquadObject:
     attrs = []
     types = None
 
+    def __init__(self, _id=None):
+        if _id:
+            self.endpoint += str(_id)
+            self.__fetch__()
+
     @classmethod
     def get_type(cls, _type):
         if SquadObject.types is None:
@@ -34,19 +38,29 @@ class SquadObject:
 
     @property
     def __id__(self):
-        return self.id if 'id' in self.attrs else uuid.uuid1()
+        if not hasattr(self, 'id'):
+            setattr(self, 'id', None)
+
+        if self.id is None:
+            self.id = uuid.uuid1()
+
+        return self.id
+
+    def __fill_object__(self, result, obj=None):
+
+        if obj is None:
+            obj = self
+
+        attrs = obj.attrs if len(obj.attrs) else [attr for attr in result.keys()]
+        for attr in attrs:
+            setattr(obj, attr.replace(' ', '_').replace('/', '_').replace('-', '_'), result[attr])
 
     def __fill__(self, klass, results):
+
         objects = {}
         for result in results:
             obj = klass()
-            attrs = klass.attrs if len(klass.attrs) else [attr.replace(' ', '_') for attr in result.keys()]
-            for attr in attrs:
-                try:
-                    setattr(obj, attr, result[attr])
-                except (AttributeError, KeyError) as e:
-                    print(e)
-                    print(attr)
+            self.__fill_object__(result, obj)
             objects[obj.__id__] = obj
 
         return objects
@@ -59,13 +73,19 @@ class SquadObject:
 
         return '%s(%s)' % (class_name, ', '.join(attrs_str))
 
-    def __fetch__(self, klass, filters, count):
+    def __fetch__(self, klass=None, filters=None, count=DEFAULT_COUNT):
         """
             Generic get method to retrieve objects from API
             count: number of objects to fetch, defaults to 50,
                       -1 (ALL) means follow pagination
         """
 
+        if klass is None:
+            response = SquadApi.get(self.endpoint)
+            result = response.json()
+            self.__fill_object__(result)
+            return
+
         if count == ALL:
             count = settings.MAX_NUM_OF_OBJECTS
 
@@ -144,6 +164,42 @@ class Squad(SquadObject):
     def reports(self, count=DEFAULT_COUNT, **filters):
         return self.__fetch__(Report, filters, count)
 
+    def submit(self, group=None, project=None, build=None, environment=None,
+               tests=None, metrics=None, metadata=None, log=None, attachments=None):
+
+        path = '/api/submit/%s/%s/%s/%s' % (group.slug, project.slug, build.version, environment.slug)
+        num_tests = 0
+        num_metrics = 0
+
+        data = {}
+        if tests:
+            tests_dict = {}
+            for test in tests.values():
+                if hasattr(test, 'log') and test.log is not None and len(test.log):
+                    value = {'log': test.log, 'result': test.status}
+                else:
+                    value = test.status
+                tests_dict[test.name] = value
+            num_tests = len(tests_dict)
+            data['tests'] = to_json(tests_dict)
+        if metrics:
+            metrics_dict = {metric.name: metric.result for metric in metrics.values()}
+            num_metrics = len(metrics_dict)
+            data['metrics'] = to_json(metrics_dict)
+        if metadata:
+            data['metadata'] = to_json(metadata)
+        if log:
+            data['log'] = log
+
+        logger.info('Submitting %i tests, %i metrics' % (num_tests, num_metrics))
+
+        # TODO handle attachments
+        response = SquadApi.post(path, data=data)
+        status_code = response.status_code
+        if status_code not in [200, 201, 500]:
+            logger.error('Failed to submit results: %s' % response.text)
+        return response.ok
+
 
 class Group(SquadObject):
 
@@ -187,7 +243,7 @@ class Project(SquadObject):
 class Build(SquadObject):
 
     endpoint = '/api/builds/'
-    attrs = ['url', 'id', 'testjobs', 'status', 'finished',
+    attrs = ['url', 'id', 'testjobs', 'finished',
              'version', 'created_at', 'datetime', 'patch_id', 'keep_data', 'project',
              'patch_source', 'patch_baseline']
 
@@ -202,6 +258,7 @@ class Build(SquadObject):
         return testruns
 
     __metadata__ = None
+    __status__ = None
 
     @property
     def metadata(self):
@@ -212,6 +269,15 @@ class Build(SquadObject):
             self.__metadata__ = first(objects)
         return self.__metadata__
 
+    @property
+    def status(self):
+        if self.__status__ is None:
+            endpoint = '%s%d/status' % (self.endpoint, self.id)
+            response = SquadApi.get(endpoint)
+            objects = self.__fill__(BuildStatus, [response.json()])
+            self.__status__ = first(objects)
+        return self.__status__
+
     def __repr__(self):
         return self.version
 
@@ -220,6 +286,10 @@ class BuildMetadata(SquadObject):
     pass
 
 
+class BuildStatus(SquadObject):
+    pass
+
+
 class TestJob(SquadObject):
 
     endpoint = '/api/testjobs/'
@@ -230,16 +300,44 @@ class TestJob(SquadObject):
              'parent_job']
 
 
+class MetricSuite:
+    name = ''
+    __metrics__ = {}
+
+    def add_metric(self, metric):
+        if self.__metric__ is None:
+            self.__metric__ = {}
+        self.__metrics__[metric.id] = metric
+
+    @property
+    def metrics(self):
+        return self.__metrics__
+
+
+class Metric(SquadObject):
+    pass
+
+
+class TestRunStatus(SquadObject):
+    attrs = ['id', 'tests_pass', 'tests_fail', 'tests_xfail',
+             'tests_skip', 'metrics_summary', 'has_metrics',
+             'suite', 'suite_version']
+
+
 class TestRun(SquadObject):
 
     endpoint = '/api/testruns/'
     attrs = ['url', 'id', 'metadata_file', 'log_file',
              'created_at', 'completed', 'datetime', 'build_url',
-             'job_id', 'job_status', 'job_url', 'resubmit_url', 'data_processed',
-             'status_recorded', 'build', 'environment']
+             'job_id', 'job_status', 'job_url', 'resubmit_url',
+             'data_processed', 'status_recorded', 'build',
+             'environment']
+    metadata = None
+    attachments = None
+    log = None
 
     def __setattr__(self, attr, value):
-        if attr == 'environment' and value.startswith('http'):
+        if attr == 'environment' and type(value) is str and value.startswith('http'):
             response = SquadApi.get(value)
             objs = self.__fill__(Environment, [response.json()])
             value = first(objs)
@@ -247,58 +345,82 @@ class TestRun(SquadObject):
 
     __tests__ = None
 
-    def tests(self, count=ALL, **filters):
+    def add_test(self, test):
         if self.__tests__ is None:
+            self.__tests__ = {}
+        self.__tests__[test.__id__] = test
+
+    def tests(self, count=ALL, **filters):
+        if self.__tests__ is None and hasattr(self, 'id') and self.id is not None:
             filters.update({'test_run': self.id})
             self.__tests__ = self.__fetch__(Test, filters, count)
         return self.__tests__
 
-    class Metric(SquadObject):
-        pass
-
     __metrics__ = None
 
-    def metrics(self, count=ALL, **filters):
+    def add_metric(self, metric):
         if self.__metrics__ is None:
-            TestRun.Metric.endpoint = '%s%d/metrics' % (self.endpoint, self.id)
-            self.__metrics__ = self.__fetch__(TestRun.Metric, filters, count)
+            self.__metrics__ = {}
+        self.__metrics__[metric.__id__] = metric
+
+    def metrics(self, count=ALL, **filters):
+        if self.__metrics__ is None and hasattr(self, 'id') and self.id is not None:
+            Metric.endpoint = '%s%d/metrics' % (self.endpoint, self.id)
+            self.__metrics__ = self.__fetch__(Metric, filters, count)
         return self.__metrics__
 
     test_suites = []
     metric_suites = []
 
-    class TestSuite:
-        name = ''
-        tests = []
-
-    class MetricSuite:
-        name = ''
-        metrics = []
-
     def bucket_metric_and_test_suites(self):
         all_tests = self.tests()
         self.test_suites = []
         if len(all_tests):
             for suite_name, tests in groupby(sorted(all_tests.values(), key=lambda t: t.name), lambda t: parse_test_name(t.name)[0]):
-                test_suite = TestRun.TestSuite()
+                test_suite = Suite()
                 test_suite.name = suite_name
-                test_suite.tests = [t for t in tests]
                 self.test_suites.append(test_suite)
+                for test in tests:
+                    test_suite.add_test(test)
 
         all_metrics = self.metrics()
         if len(all_metrics):
             self.metric_suites = []
             for suite_name, metrics in groupby(sorted(all_metrics.values(), key=lambda m: m.name), lambda m: parse_metric_name(m.name)[0]):
-                metric_suite = TestRun.MetricSuite()
+                metric_suite = MetricSuite()
                 metric_suite.name = suite_name
-                metric_suite.metrics = [m for m in metrics]
+                [metric_suite.add_metric(m) for m in metrics]
                 self.metric_suites.append(metric_suite)
 
+    def submit_results(self):
+        squad = Squad()
+        return squad.submit(
+            group=self.build.project.group,
+            project=self.build.project,
+            build=self.build,
+            environment=self.environment,
+            tests=self.tests(),
+            metrics=self.metrics(),
+            metadata=self.metadata,
+            log=self.log,
+            attachments=self.attachments)
+
+    __summary__ = None
+
+    def summary(self):
+        if self.__summary__ is None:
+            self.__summary__ = first(self.statuses(suite__isnull=True))
+        return self.__summary__
+
+    def statuses(self, count=ALL, **filters):
+        TestRunStatus.endpoint = '%s%d/status' % (self.endpoint, self.id)
+        return self.__fetch__(TestRunStatus, filters, count)
+
 
 class Test(SquadObject):
 
     endpoint = '/api/tests/'
-    attrs = ['id', 'name', 'short_name', 'status', 'result', 'log', 'has_known_issues',
+    attrs = ['id', 'name', 'short_name', 'status', 'result', 'test_run', 'log', 'has_known_issues',
              'suite', 'known_issues']
 
 
@@ -307,6 +429,17 @@ class Suite(SquadObject):
     endpoint = '/api/suites/'
     attrs = ['id', 'slug', 'name', 'project']
 
+    __tests__ = {}
+
+    def add_test(self, test):
+        if self.__tests__ is None:
+            self.__tests__ = {}
+        self.__tests__[test.id] = test
+
+    @property
+    def tests(self):
+        return self.__tests__
+
 
 class Environment(SquadObject):
 
@@ -348,7 +481,7 @@ class Annotation(SquadObject):
 class MetricThreshold(SquadObject):
 
     endpoint = '/api/metricthresholds/'
-    attrs = ['url', 'id', 'name', 'value', 'is_higher_better', 'project']
+    attrs = ['url', 'id', 'name', 'value', 'is_higher_better', 'environment']
 
 
 class Report(SquadObject):
